@@ -9,7 +9,14 @@ from sqlglot.errors import ParseError
 from sqlglot.helper import AutoName, camel_to_snake_case, ensure_list
 
 
-class Expression:
+class _Expression(type):
+    def __new__(cls, clsname, bases, attrs):
+        klass = super().__new__(cls, clsname, bases, attrs)
+        klass.key = clsname.lower()
+        return klass
+
+
+class Expression(metaclass=_Expression):
     """
     The base class for all expressions in a syntax tree.
 
@@ -21,11 +28,11 @@ class Expression:
             or optional (False).
     """
 
+    key = None
     arg_types = {"this": True}
-    __slots__ = ("key", "args", "parent", "arg_key")
+    __slots__ = ("args", "parent", "arg_key")
 
     def __init__(self, **args):
-        self.key = self.__class__.__name__.lower()
         self.args = args
         self._set_parent(args)
         self.parent = None
@@ -573,7 +580,7 @@ class Lambda(Expression):
 
 
 class Limit(Expression):
-    pass
+    arg_types = {"this": False, "expression": True}
 
 
 class Literal(Condition):
@@ -666,11 +673,11 @@ class Lateral(Expression):
 
 
 class Offset(Expression):
-    pass
+    arg_types = {"this": False, "expression": True}
 
 
 class Order(Expression):
-    arg_types = {"expressions": True}
+    arg_types = {"this": False, "expressions": True}
 
 
 class Ordered(Expression):
@@ -791,7 +798,8 @@ class Union(Subqueryable, Expression):
     @property
     def named_selects(self):
         named_selects = (
-            self.args["this"].named_selects + self.args["expression"].named_selects
+            self.args["this"].unnest().named_selects
+            + self.args["expression"].unnest().named_selects
         )
         return sorted(set(named_selects), key=named_selects.index)
 
@@ -1307,6 +1315,15 @@ class Select(Subqueryable, Expression):
 class Subquery(Expression, DerivedTable):
     arg_types = {"this": True, "alias": False}
 
+    def unnest(self):
+        """
+        Returns the first non subquery.
+        """
+        expression = self
+        while isinstance(expression, Subquery):
+            expression = expression.this
+        return expression
+
 
 class TableSample(Expression):
     arg_types = {
@@ -1339,19 +1356,15 @@ class Where(Expression):
 
 
 class Star(Expression):
-    arg_types = {}
+    arg_types = {"except": False, "replace": False}
 
     @property
     def name(self):
         return "*"
 
 
-class StarExcept(Expression):
-    arg_types = {"this": True, "expressions": True}
-
-
-class StarReplace(Expression):
-    arg_types = {"this": True, "expressions": True}
+class Placeholder(Expression):
+    arg_types = {}
 
 
 class Null(Condition):
@@ -1610,7 +1623,21 @@ class In(Predicate):
     arg_types = {"this": True, "expressions": False, "query": False}
 
 
-class Interval(Expression):
+class TimeUnit(Expression):
+    """Automatically converts unit arg into a var."""
+
+    arg_types = {"unit": False}
+
+    def __init__(self, **args):
+        unit = args.get("unit")
+        if isinstance(unit, Column):
+            args["unit"] = Var(this=unit.name)
+        elif isinstance(unit, Week):
+            unit.set("this", Var(this=unit.this.name))
+        super().__init__(**args)
+
+
+class Interval(TimeUnit):
     arg_types = {"this": True, "unit": False}
 
 
@@ -1763,19 +1790,47 @@ class CurrentDate(Func):
     arg_types = {"this": False}
 
 
-class DateAdd(Func):
+class CurrentTimestamp(Func):
+    arg_types = {}
+
+
+class DateAdd(Func, TimeUnit):
     arg_types = {"this": True, "expression": True, "unit": False}
 
 
-class DateDiff(Func):
+class DateDiff(Func, TimeUnit):
     arg_types = {"this": True, "expression": True, "unit": False}
 
 
+class DatetimeDiff(Func, TimeUnit):
+    arg_types = {"this": True, "expression": True, "unit": False}
+
+
+class TimestampDiff(Func, TimeUnit):
+    arg_types = {"this": True, "expression": True, "unit": False}
+
+
+class TimestampTrunc(Func, TimeUnit):
+    arg_types = {"this": True, "unit": True, "zone": False}
+
+
+class DateTrunc(Func, TimeUnit):
+    arg_types = {"this": True, "unit": True, "zone": False}
+
+
+class DatetimeTrunc(Func, TimeUnit):
+    arg_types = {"this": True, "unit": True, "zone": False}
+
+
+class TimeTrunc(Func, TimeUnit):
+    arg_types = {"this": True, "unit": True, "zone": False}
+
+
 class DateStrToDate(Func):
     pass
 
 
-class DateSub(Func):
+class DateSub(Func, TimeUnit):
     arg_types = {"this": True, "expression": True, "unit": False}
 
 
@@ -1987,7 +2042,7 @@ class TimeStrToUnix(Func):
     pass
 
 
-class TsOrDsAdd(Func):
+class TsOrDsAdd(Func, TimeUnit):
     arg_types = {"this": True, "expression": True, "unit": False}
 
 
@@ -2031,6 +2086,10 @@ class VarianceSamp(AggFunc):
     pass
 
 
+class Week(Func):
+    pass
+
+
 class Year(Func):
     pass
 
