@@ -27,7 +27,10 @@ class Parser:
         name: f.from_arg_list for f in exp.ALL_FUNCTIONS for name in f.sql_names()
     }
 
-    NO_PAREN_FUNCTIONS = {TokenType.CURRENT_DATE: exp.CurrentDate}
+    NO_PAREN_FUNCTIONS = {
+        TokenType.CURRENT_DATE: exp.CurrentDate,
+        TokenType.CURRENT_TIMESTAMP: exp.CurrentTimestamp,
+    }
 
     TYPE_TOKENS = {
         TokenType.BOOLEAN,
@@ -112,6 +115,7 @@ class Parser:
 
     FUNC_TOKENS = {
         TokenType.CURRENT_DATE,
+        TokenType.CURRENT_TIMESTAMP,
         TokenType.EXTRACT,
         TokenType.OFFSET,
         TokenType.PRIMARY_KEY,
@@ -415,9 +419,9 @@ class Parser:
 
     def _parse_drop(self):
         if self._match(TokenType.TABLE):
-            kind = "table"
+            kind = "TABLE"
         elif self._match(TokenType.VIEW):
-            kind = "view"
+            kind = "VIEW"
         else:
             self.raise_error("Expected TABLE or View")
 
@@ -1009,12 +1013,12 @@ class Parser:
             return None
         return self.expression(exp.Qualify, this=self._parse_conjunction())
 
-    def _parse_order(self):
+    def _parse_order(self, this=None):
         if not self._match_by(TokenType.ORDER):
-            return None
+            return this
 
         return self.expression(
-            exp.Order, expressions=self._parse_csv(self._parse_ordered)
+            exp.Order, this=this, expressions=self._parse_csv(self._parse_ordered)
         )
 
     def _parse_ordered(self):
@@ -1022,15 +1026,15 @@ class Parser:
         self._match(TokenType.ASC)
         return self.expression(exp.Ordered, this=this, desc=self._match(TokenType.DESC))
 
-    def _parse_limit(self, top=False):
+    def _parse_limit(self, this=None, top=False):
         if not self._match(TokenType.TOP if top else TokenType.LIMIT):
-            return None
-        return self.expression(exp.Limit, this=self._parse_number())
+            return this
+        return self.expression(exp.Limit, this=this, expression=self._parse_number())
 
-    def _parse_offset(self):
+    def _parse_offset(self, this=None):
         if not self._match(TokenType.OFFSET):
-            return None
-        return self.expression(exp.Offset, this=self._parse_number())
+            return this
+        return self.expression(exp.Offset, this=this, expression=self._parse_number())
 
     def _parse_set_operations(self, this):
         if not self._match_set(self.SET_OPERATIONS):
@@ -1143,13 +1147,15 @@ class Parser:
                 unit=self._parse_var(),
             )
 
+        index = self._index
         type_token = self._parse_types()
         this = self._parse_column()
 
         if type_token:
             if this:
                 return self.expression(exp.Cast, this=this, to=type_token)
-            return type_token
+            self._retreat(index)
+            return self._parse_column()
 
         if self._match(TokenType.DCOLON):
             type_token = self._parse_types()
@@ -1191,10 +1197,8 @@ class Parser:
                 self.raise_error("Expecting >")
 
         if type_token in self.TIMESTAMPS:
-            tz = self._match(TokenType.WITH)
-            self._match(TokenType.WITHOUT)
-            self._match(TokenType.TIME)
-            self._match(TokenType.ZONE)
+            tz = self._match(TokenType.WITH_TIME_ZONE)
+            self._match(TokenType.WITHOUT_TIME_ZONE)
             if tz:
                 return exp.DataType(
                     this=exp.DataType.Type.TIMESTAMPTZ,
@@ -1244,11 +1248,12 @@ class Parser:
 
     def _parse_primary(self):
         this = (
-            self._parse_string()
-            or self._parse_number()
+            self._parse_string(False)
+            or self._parse_number(False)
             or self._parse_star()
             or self._parse_null()
             or self._parse_boolean()
+            or self._parse_placeholder()
         )
 
         if this:
@@ -1313,11 +1318,11 @@ class Parser:
             function = self.FUNCTIONS.get(this)
             args = self._parse_csv(self._parse_lambda)
 
-            if not callable(function):
-                this = self.expression(exp.Anonymous, this=this, expressions=args)
-            else:
+            if function:
                 this = function(args)
                 self.validate_expression(this, args)
+            else:
+                this = self.expression(exp.Anonymous, this=this, expressions=args)
         self._match_r_paren()
         return self._parse_window(this)
 
@@ -1344,7 +1349,7 @@ class Parser:
             else:
                 self._match(TokenType.RESPECT_NULLS)
 
-            return this
+            return self._parse_alias(self._parse_limit(self._parse_order(this)))
 
         return self.expression(
             exp.Lambda,
@@ -1585,25 +1590,25 @@ class Parser:
             this=self._prev.text, quoted=False
         )
 
-    def _parse_string(self):
+    def _parse_string(self, placeholder=True):
         if self._match(TokenType.STRING):
             return exp.Literal.string(self._prev.text)
-        return None
+        return self._parse_placeholder() if placeholder else None
 
-    def _parse_number(self):
+    def _parse_number(self, placeholder=True):
         if self._match(TokenType.NUMBER):
             return exp.Literal.number(self._prev.text)
-        return None
+        return self._parse_placeholder() if placeholder else None
 
-    def _parse_identifier(self):
+    def _parse_identifier(self, placeholder=True):
         if self._match(TokenType.IDENTIFIER):
             return exp.Identifier(this=self._prev.text, quoted=True)
-        return None
+        return self._parse_placeholder() if placeholder else None
 
-    def _parse_var(self):
+    def _parse_var(self, placeholder=True):
         if self._match(TokenType.VAR):
             return exp.Var(this=self._prev.text)
-        return None
+        return self._parse_placeholder() if placeholder else None
 
     def _parse_null(self):
         if self._match(TokenType.NULL):
@@ -1619,26 +1624,33 @@ class Parser:
 
     def _parse_star(self):
         if self._match(TokenType.STAR):
-            return self._parse_replace(self._parse_except(exp.Star()))
+            return exp.Star(
+                **{"except": self._parse_except(), "replace": self._parse_replace()}
+            )
         return None
 
-    def _parse_except(self, this):
+    def _parse_placeholder(self):
+        if self._match(TokenType.PLACEHOLDER):
+            return exp.Placeholder()
+        return None
+
+    def _parse_except(self):
         if not self._match(TokenType.EXCEPT):
-            return this
+            return None
 
         self._match_l_paren()
         columns = self._parse_csv(self._parse_id_var)
         self._match_r_paren()
-        return self.expression(exp.StarExcept, this=this, expressions=columns)
+        return columns
 
-    def _parse_replace(self, this):
+    def _parse_replace(self):
         if not self._match(TokenType.REPLACE):
-            return this
+            return None
 
         self._match_l_paren()
         columns = self._parse_csv(lambda: self._parse_alias(self._parse_id_var()))
         self._match_r_paren()
-        return self.expression(exp.StarReplace, this=this, expressions=columns)
+        return columns
 
     def _parse_csv(self, parse):
         parse_result = parse()
