@@ -1,6 +1,6 @@
 from enum import auto
 
-from sqlglot.helper import AutoName, list_get
+from sqlglot.helper import AutoName
 from sqlglot.trie import in_trie, new_trie
 
 
@@ -100,6 +100,7 @@ class TokenType(AutoName):
     CROSS = auto()
     CURRENT_DATE = auto()
     CURRENT_ROW = auto()
+    CURRENT_TIMESTAMP = auto()
     DIV = auto()
     DEFAULT = auto()
     DELETE = auto()
@@ -155,6 +156,7 @@ class TokenType(AutoName):
     OVERWRITE = auto()
     PARTITION = auto()
     PERCENT = auto()
+    PLACEHOLDER = auto()
     PRECEDING = auto()
     PRIMARY_KEY = auto()
     PROPERTIES = auto()
@@ -192,9 +194,9 @@ class TokenType(AutoName):
     WHEN = auto()
     WHERE = auto()
     WITH = auto()
+    WITH_TIME_ZONE = auto()
     WITHIN_GROUP = auto()
-    WITHOUT = auto()
-    ZONE = auto()
+    WITHOUT_TIME_ZONE = auto()
 
 
 class Token:
@@ -231,11 +233,16 @@ class _Tokenizer(type):
     def __new__(cls, clsname, bases, attrs):
         klass = super().__new__(cls, clsname, bases, attrs)
 
-        quotes = {quote: TokenType.QUOTE for quote in klass.QUOTES}
+        klass.QUOTES = [
+            prefix + quote for prefix in ["", "r"] for quote in klass.QUOTES
+        ]
 
         klass.AMBIGUOUS = new_trie(
-            key
-            for key, value in {**klass.KEYWORDS, **quotes}.items()
+            key.upper()
+            for key, value in {
+                **klass.KEYWORDS,
+                **{quote: TokenType.QUOTE for quote in klass.QUOTES},
+            }.items()
             if value in (TokenType.COMMENT, TokenType.COMMENT_START, TokenType.QUOTE)
             or " " in key
             or any(single in key for single in klass.SINGLE_TOKENS)
@@ -269,6 +276,7 @@ class Tokenizer(metaclass=_Tokenizer):
         "/": TokenType.SLASH,
         "*": TokenType.STAR,
         "~": TokenType.TILDA,
+        "?": TokenType.PLACEHOLDER,
     }
 
     QUOTES = ["'"]
@@ -319,6 +327,7 @@ class Tokenizer(metaclass=_Tokenizer):
         "CROSS": TokenType.CROSS,
         "CURRENT_DATE": TokenType.CURRENT_DATE,
         "CURRENT ROW": TokenType.CURRENT_ROW,
+        "CURRENT_TIMESTAMP": TokenType.CURRENT_TIMESTAMP,
         "DIV": TokenType.DIV,
         "DEFAULT": TokenType.DEFAULT,
         "DELETE": TokenType.DELETE,
@@ -394,7 +403,6 @@ class Tokenizer(metaclass=_Tokenizer):
         "TEMP": TokenType.TEMPORARY,
         "TEMPORARY": TokenType.TEMPORARY,
         "THEN": TokenType.THEN,
-        "TIME": TokenType.TIME,
         "TOP": TokenType.TOP,
         "TRUE": TokenType.TRUE,
         "TRUNCATE": TokenType.TRUNCATE,
@@ -410,9 +418,9 @@ class Tokenizer(metaclass=_Tokenizer):
         "WHEN": TokenType.WHEN,
         "WHERE": TokenType.WHERE,
         "WITH": TokenType.WITH,
+        "WITH TIME ZONE": TokenType.WITH_TIME_ZONE,
         "WITHIN GROUP": TokenType.WITHIN_GROUP,
-        "WITHOUT": TokenType.WITHOUT,
-        "ZONE": TokenType.ZONE,
+        "WITHOUT TIME ZONE": TokenType.WITHOUT_TIME_ZONE,
         "ARRAY": TokenType.ARRAY,
         "BOOL": TokenType.BOOLEAN,
         "BOOLEAN": TokenType.BOOLEAN,
@@ -494,7 +502,6 @@ class Tokenizer(metaclass=_Tokenizer):
         "_char",
         "_end",
         "_peek",
-        "__text",
     )
 
     def __init__(
@@ -525,25 +532,22 @@ class Tokenizer(metaclass=_Tokenizer):
         self._char = None
         self._end = None
         self._peek = None
-        self.__text = None
 
     def tokenize(self, sql):
         self.reset()
         self.sql = sql
         self.size = len(sql)
 
-        while not self._end:
+        while self.size and not self._end:
             self._start = self._current
             self._advance()
 
             if not self._char:
                 break
-            if self._scan_ambiguous():
-                pass
-            elif self._char in self.SINGLE_TOKENS:
-                self._add(self.SINGLE_TOKENS[self._char])
-            elif self._char in self.WHITE_SPACE:
-                white_space = self.WHITE_SPACE[self._char]
+
+            white_space = self.WHITE_SPACE.get(self._char)
+
+            if white_space:
                 if white_space == TokenType.BREAK:
                     self._col = 1
                     self._line += 1
@@ -553,6 +557,10 @@ class Tokenizer(metaclass=_Tokenizer):
                 self._scan_identifier()
             elif self._char == "#":
                 self._scan_annotation()
+            elif self._scan_ambiguous():
+                pass
+            elif self._char in self.SINGLE_TOKENS:
+                self._add(self.SINGLE_TOKENS[self._char])
             else:
                 self._scan_var()
         return self.tokens
@@ -569,16 +577,13 @@ class Tokenizer(metaclass=_Tokenizer):
     def _advance(self, i=1):
         self._col += i
         self._current += i
-        self._char = list_get(self.sql, self._current - 1)
-        self._peek = list_get(self.sql, self._current) or ""
         self._end = self._current >= self.size
-        self.__text = None
+        self._char = self.sql[self._current - 1]
+        self._peek = self.sql[self._current] if self._current < self.size else ""
 
     @property
     def _text(self):
-        if self.__text is None:
-            self.__text = self.sql[self._start : self._current]
-        return self.__text
+        return self.sql[self._start : self._current]
 
     def _add(self, token_type, text=None):
         text = self._text if text is None else text
@@ -596,7 +601,7 @@ class Tokenizer(metaclass=_Tokenizer):
     def _scan_ambiguous(self):
         size = 1
         word = None
-        chars = self._chars(size)
+        chars = self._text
 
         while chars:
             result = in_trie(self.AMBIGUOUS, chars.upper())
@@ -618,7 +623,7 @@ class Tokenizer(metaclass=_Tokenizer):
             return True
 
         self._advance(len(word) - 1)
-        self._add(self.KEYWORDS[word])
+        self._add(self.KEYWORDS[word.upper()])
         return True
 
     def _scan_comment(self, comment):
@@ -679,33 +684,34 @@ class Tokenizer(metaclass=_Tokenizer):
             return False
 
         size = len(quote)
-        text = list(self._chars(size))
+        text = ""
         self._advance(size)
 
+        if quote.startswith("r"):
+            quote = quote[1:]
+            size = len(quote)
+
         while True:
             if self._char == self.escape and self._peek == quote:
-                text.extend((self.ESCAPE_CODE, self._char))
+                text += f"{self.ESCAPE_CODE}{self._char}"
                 self._advance(2)
             else:
-                chars = self._chars(size)
-
-                if chars == quote:
-                    text.extend(chars)
-                    self._advance(size - 1)
+                if self._chars(size) == quote:
+                    if size > 1:
+                        self._advance(size - 1)
                     break
 
                 if self._char == "'":
-                    text.extend((self.ESCAPE_CODE, self._char))
+                    text += f"{self.ESCAPE_CODE}{self._char}"
                     self._advance()
+                elif self._end:
+                    raise RuntimeError(
+                        f"Missing {quote} from {self._line}:{self._start}"
+                    )
                 else:
-                    if self._end:
-                        raise RuntimeError(
-                            f"Missing {quote} from {self._line}:{self._start}"
-                        )
-                    text.append(self._char)
+                    text += self._char
                     self._advance()
 
-        text = "".join(text[size:-size])
         text = text.encode(self.ENCODE).decode(self.ENCODE) if self.ENCODE else text
         text = text.replace("\\\\", "\\") if self.escape == "\\" else text
         self._add(TokenType.STRING, text)
